#!/usr/local/bin/ruby
#
# Copyright (c) Yasuhiro ABE <yasu@dengaku.org>
#

require "rnids"

class PATIMON < NIDS
  def detected(pkt)
    printf "!! detected at [%s] !!\n" , Time.now
    printf "%s:%d -> %s:%d\n", 
      pkt.saddr, pkt.sport,
      pkt.daddr, pkt.dport
  end

  def callback(pkt)
    return if not pkt.kind_of?(NIDS_TCP)
    case pkt.tcp_state
    when NIDS::NIDS_JUST_EST
    when NIDS::NIDS_DATA
      if @tcp_sigs[String(pkt.dport)]
	if @tcp_sigs[String(pkt.dport)][String(pkt.sport)]
	  @tcp_sigs[String(pkt.dport)][String(pkt.sport)].each{ |sig|
	    detected(pkt) if @tcp_sigs_buf + pkt.data =~ sig
	  }
	else
	  @tcp_sigs[String(pkt.dport)]["*"].each{ |sig|
	    detected(pkt) if @tcp_sigs_buf + pkt.data =~ sig
	  }
	end
      else
	if @tcp_sigs["*"][String(pkt.sport)]
	  @tcp_sigs["*"][String(pkt.sport)].each{ |sig|
	    detected(pkt) if @tcp_sigs_buf + pkt.data =~ sig
	  }
	else
	  @tcp_sigs["*"]["*"].each{ |sig|
	    detected(pkt) if @tcp_sigs_buf + pkt.data =~ sig
	  }
	end
      end
      @tcp_sigs_buf = pkt.data[pkt.data.size - @tcp_sigs_maxlen,30].to_s
    when NIDS::NIDS_CLOSE
    when NIDS::NIDS_RESET
    when NIDS::NIDS_TIMED_OUT
    else
      raise "you don't allow reach here"
    end
  end

  def read_sigs(file)
    @tcp_sigs_maxlen = 0
    @tcp_sigs_buf = ""
    @tcp_sigs = Hash.new
    if FileTest.exist?(file)
      open(file).each_line{|line|
	next if line == "" or line == nil
	next if line =~ /^\#/
	name,proto,sp,dp,sig = line.split(" ",5)
	next if name == nil
	next if proto != "tcp"
	#
	# convert 'sig' to 'ruby-sig'
	rsig = sig.chop.chop
	rsig = rsig[1,rsig.size-1]

	if rsig =~ "\\|" #or rsig =~ "%"
	  # HINT: 's="ab";s[0]=Integer("0xa4");s[1]=Integer("0xa2"); print s'
	  # HINT: 's="ab";s[0]="a4".hex;s[1]="a2".hex; print s'
	  # HINT: '|'ƬʸŤbyteѴ
	  #       ˡɤʡ
	  h_in = false
	  rrsig = ""
	  rsig.each_byte{ |b|
	    if h_in 
	      if b == ' '
		p rrsig.dump
		next 
	      end
	      rrsig << b
	      next
	    end

	    if b == '|' and h_in
	      h_in = false
	    elsif b == '|' 
	      h_in = true
	    end
	  }
	  if $DEBUG
	    print "Sorry, this signature is not acceptable\n"
	    print "=> " + rsig + "\n"
	  end
	  next
	end

	# produce 'ruby-sig' form
	if @tcp_sigs[dp] == nil
	  @tcp_sigs[dp] = {sp=>rsig}
	else
	  if @tcp_sigs[dp][sp] == nil
	    @tcp_sigs[dp][sp] = rsig
	  else
	    @tcp_sigs[dp][sp] = [rsig,@tcp_sigs[dp][sp]]
	  end
	end
	
	@tcp_sigs_maxlen = @tcp_sigs_maxlen < rsig.size ? rsig.size : @tcp_sigs_maxlen
	if $DEBUG
	  print rsig + "\n"
	end
      }
    else
      raise "cannot read the file #{file}"
    end
    if $DEBUG
      p @tcp_sigs.each{|key,value|
	print key
	print " => "
	p value
      }
    end
  end
end

#
# MAIN proc
#

patimon = PATIMON.new
patimon.read_sigs(ARGV[0])
patimon.use_kill false
#patimon.run # for lan!

# end
