#!/usr/local/bin/ruby
#
# patemon.rb (aka. patimon.rb) : compatible with a pakemon rules file.
#   Note: The patemon.rb needs more power, which has been never seen. 
#
# Copyright (c) 2001 Yasuhiro ABE <yasu@dengaku.org>
# Version: 0.5477225575 (still alpha version)
#
#
# usage(): ruby patemon.rb <signature file>
#
# e.x.: sudo ruby examples/patemon.rb visicon-pakemon.conf
#

require "rnids"

class PATIMON < NIDS

  def setup
    # @logger = open("patemon.out.logger", "w")
    @logger = $stdout
    return self
  end

  def detected(pkt,sig)
    @logger.printf "!! hostile action is detected by %s\n", sig.source
    @logger.printf "!! time is  %s \n", Time.now.strftime("%X") + "." + Time.now.usec.to_s
    @logger.printf "!! %s:%4d -> %s:%4d (%s)\n", 
      pkt.saddr, pkt.sport,
      pkt.daddr, pkt.dport, pkt.tcp_direction?
    @tcp_sigs_buf = pkt.data[pkt.data_len-@tcp_sigs_maxlen,@tcp_sigs_maxlen].to_s
    #
    # shutdown this session
    @logger.flush
  end

  def callback(pkt)
    return if not pkt.kind_of?(NIDS_TCP)
    case pkt.tcp_state
    when NIDS::NIDS_JUST_EST
    when NIDS::NIDS_DATA
      if @tcp_sigs[String(pkt.dport)]
	if @tcp_sigs[String(pkt.dport)][String(pkt.sport)]
	  @tcp_sigs[String(pkt.dport)][String(pkt.sport)].each{ |sig|	    
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	elsif @tcp_sigs[String(pkt.dport)]["*"]
	  @tcp_sigs[String(pkt.dport)]["*"].each{ |sig|
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	end
      end
      if @tcp_sigs["*"]
	if @tcp_sigs["*"][String(pkt.sport)]
	  @tcp_sigs["*"][String(pkt.sport)].each{ |sig|
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	end
	if @tcp_sigs["*"]["*"]
	  @tcp_sigs["*"]["*"].each{ |sig|
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	end
      end
    when NIDS::NIDS_CLOSE
    when NIDS::NIDS_RESET
    when NIDS::NIDS_TIMED_OUT
    else
      raise "you don't allow reach here"
    end
  end

  def read_sigs(file)
    @tcp_sigs_maxlen = 0
    @tcp_sigs_buf = ""
    @tcp_sigs = Hash.new
    if FileTest.exist?(file)
      open(file).each_line{|line|
	next if line == "" or line == nil
	next if line =~ /^\#/
	name,proto,sp,dp,sig = line.split(" ",5)
	next if name == nil
	next if proto != "tcp"
	#
	# convert 'sig' to 'ruby-sig'
	sig.strip! # remove useless spaces
	rsig = sig.chop
	rsig = rsig[1,rsig.size-1]

	# when signature contains the hex format strings
	if rsig =~ "\\|" 
	  tmp_sig = ""
	  tmp_sig_pos = 0
	  b = rsig.index('|',0)
	  e = rsig.index('|',b+1)

	  if b != 0
	    rsig[0..b-1].each_byte{|byte|
	      tmp_sig += " "
	      tmp_sig[tmp_sig_pos] = byte
	      tmp_sig_pos += 1
	    }
	  end
	  hex_sig = rsig[b+1..e-1].gsub(" ","")
	  i=0
	  while hex_sig != nil and i < hex_sig.size
	    if hex_sig[i] and hex_sig[i+1]
	      s = "  "
	      s[0] = hex_sig[i]
	      s[1] = hex_sig[i+1]
	      
	      tmp_sig += " "
	      tmp_sig[tmp_sig_pos] = s.hex
	      tmp_sig_pos += 1
	    end
	    i+=2
	  end

	  # next process
	  while e != rsig.size
	    b = rsig.index('|', e+1) # search next '|'
	    if b != nil # in the case, letters 'b' to next '|' is clear text.
	      rsig[e+1..b-1].each_byte{|byte|
		tmp_sig += " "
		tmp_sig[tmp_sig_pos] = byte
		tmp_sig_pos += 1
	      }

	      e = rsig.index('|', b+1)
	      hex_sig = rsig[b+1..e-1].gsub(" ","")
	      i=0
	      while hex_sig != nil and i < hex_sig.size
		if hex_sig[i] and hex_sig[i+1]
		  s = "  "
		  s[0] = hex_sig[i]
		  s[1] = hex_sig[i+1]
		  
		  tmp_sig += " "
		  tmp_sig[tmp_sig_pos] = s.hex
		  tmp_sig_pos += 1
		end
		i+=2
	      end
	    else
	      # there is no more '|' within left letters
	      rsig[e+1..rsig.size-1].each_byte{|byte|
		tmp_sig += " "
		tmp_sig[tmp_sig_pos] = byte
		tmp_sig_pos += 1
	      }
	      break
	    end
	  end
	end # finish (when signature contains the hex format strings)
	raise "error: wrong signature format is detected" if tmp_sig == ""
	rsig = tmp_sig if tmp_sig != nil 

	# produce quoted signature strings
	rsig = Regexp::new(Regexp::escape(rsig))
	# produce 'ruby-sig' form
	if @tcp_sigs[dp] == nil
	  @tcp_sigs[dp] = {sp=>[rsig]}
	else
	  if @tcp_sigs[dp][sp] == nil
	    @tcp_sigs[dp][sp] = [rsig]
	  else
	    @tcp_sigs[dp][sp] << rsig
	  end
	end
	
	@tcp_sigs_maxlen = @tcp_sigs_maxlen < rsig.source.size ? rsig.source.size : @tcp_sigs_maxlen
	if $DEBUG
	  print rsig.source + "\n"
	end
      }
    else
      raise "cannot read the file #{file}"
    end
  end
end

#
# MAIN proc
#

def usage()
  printf " welcome patemon world\n"
  printf " usage: #{$0} <rule file>\n"
  printf " <rule file>: pakemon signature file compatible\n"
  printf " note: you need root privilege\n\n"
  exit
end

if ARGV[0] == nil 
  print "no input file\n\n"
  usage
end

if ! FileTest.exist?(ARGV[0])
  ptint "wrong input file\n\n"
end

patimon = PATIMON.new.setup
patimon.read_sigs(ARGV[0])
patimon.use_kill false
patimon.run # for lan!

# end

