#!/usr/local/bin/ruby
#
# patemon.rb (aka. patimon.rb) : compatible with a pakemon rules file.
#   Note: The patemon.rb needs more power, which has been never seen. 
# Version: 0.5477225575
# 
# Copyright (C) 2000,2001 Yasuhiro ABE <yasu@dengaku.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# usage(): ruby patemon.rb <rule file>
#
# e.x.: sudo ruby examples/patemon.rb visicon-pakemon.conf
#  (The "sudo" can execute a command as root)
#

require "rnids"

class PATIMON < NIDS

  def setup
#    @logger = open("patemon.out.logger", "w")
    @logger = $stdout
    return self
  end

  def detected(pkt,sig)
    @logger.printf "!! hostile action is detected by %s\n", sig.source.dump
    @logger.printf "!! time is  %s \n", Time.now.strftime("%X") + "." + Time.now.usec.to_s
    @logger.printf "!! %s:%4d -> %s:%4d (%s)\n", 
      pkt.saddr, pkt.sport,
      pkt.daddr, pkt.dport, pkt.tcp_direction?
    @tcp_sigs_buf = pkt.data[pkt.data_len-@tcp_sigs_maxlen,@tcp_sigs_maxlen].to_s
    #
    # shutdown this session
    #pkt.kill if pkt.kind_of?(NIDS_TCP)
    @logger.flush
  end

  def callback(pkt)
    # @logger.printf "pkt(%d,%s) alive at %s\n", pkt.dport,pkt.tcp_direction?,Time.now.strftime("%X") + "." + Time.now.usec.to_s
    return if not pkt.kind_of?(NIDS_TCP)
    case pkt.tcp_state
    when NIDS_JUST_EST
    when NIDS_DATA
      @renge_sigs.each_pair{ |dport,hash|
	b_dport,e_dport = dport.split(":",2)
	if (b_dport.to_i <= pkt.dport and pkt.dport <= e_dport.to_i) or
	    (e_dport == "" and b_dport.to_i <= pkt.dport) or
	    (b_dport == "" and pkt.dport <= e_dport.to_i)
	  if hash[String(pkt.sport)]
	    hash[String(pkt.sport)].each{|sig|
	      if pkt.data =~ sig
		detected(pkt,sig)
		return
	      end
	    }
	  else
	    hash["*"].each{|sig|
	      if pkt.data =~ sig
		detected(pkt,sig)
		return
	      end
	    }
	  end
	end
      }
      if @tcp_sigs[String(pkt.dport)]
	if @tcp_sigs[String(pkt.dport)][String(pkt.sport)]
	  if $DEBUG
	    printf "processing @tcp_sigs[%d][%d]: %s\n", String(pkt.dport),
	      String(pkt.sport), @tcp_sigs[String(pkt.dport)][String(pkt.sport)]
	  end
	  @tcp_sigs[String(pkt.dport)][String(pkt.sport)].each{ |sig|	    
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	elsif @tcp_sigs[String(pkt.dport)]["*"]
	  if $DEBUG
	    printf "processing @tcp_sigs[%d][\"*\"]: %s\n", String(pkt.dport),
	      @tcp_sigs[String(pkt.dport)]["*"]
	  end
	  @tcp_sigs[String(pkt.dport)]["*"].each{ |sig|
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	end
      end
      if @tcp_sigs["*"]
	if @tcp_sigs["*"][String(pkt.sport)]
	  if $DEBUG
	    printf "processing @tcp_sigs[\"*\"][%d]: %s\n", String(pkt.sport),
	      @tcp_sigs["*"][String(pkt.sport)]
	  end
	  @tcp_sigs["*"][String(pkt.sport)].each{ |sig|
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	end
	if @tcp_sigs["*"]["*"]
	  if $DEBUG
	    printf "processing @tcp_sigs[\"*\"][\"*\"]: %s\n",
	      @tcp_sigs["*"]["*"]
	  end
	  @tcp_sigs["*"]["*"].each{ |sig|
	    if pkt.data =~ sig
	      detected(pkt,sig)
	      return
	    end
	  }
	end
      end
    when NIDS_CLOSE
      @logger.printf "!! CLOSE: %s:%4d -> %s:%4d\n", 
	pkt.saddr, pkt.sport,
	pkt.daddr, pkt.dport
    when NIDS_RESET
      @logger.printf "!! RESET: %s:%4d -> %s:%4d\n", 
	pkt.saddr, pkt.sport,
	pkt.daddr, pkt.dport
    when NIDS_TIMED_OUT
      @logger.printf "!! TIMED_OUT: %s:%4d -> %s:%4d\n", 
	pkt.saddr, pkt.sport,
	pkt.daddr, pkt.dport
    else
      raise "you don't allow reach here"
    end
  end

  def read_sigs(file)
    @tcp_sigs_maxlen = 0
    @tcp_sigs_buf = ""
    @tcp_sigs = Hash.new
    @renge_sigs = Hash.new
    if FileTest.exist?(file)
      open(file).each_line{|line|
	next if line == "" or line == nil
	next if line =~ /^\#/
	name,proto,sp,dp,sig = line.split(" ",5)
	next if name == nil
	next if proto != "tcp"
	#
	# convert 'sig' to 'ruby-sig'
	sig.strip! # remove useless spaces
	rsig = sig.chop
	rsig = rsig[1,rsig.size-1]

	if $DEBUG
	  printf "\nbegin: non hexadecimal signature\n"
	  printf "sig = '%s'\n", sig.strip
	  printf "end:\n"
	end

	is_renge = false
	if dp =~ ":"
	  is_renge = true
	end

	# when signature contains the hex format strings
	if rsig =~ "\\|" 
	  if $DEBUG
	    printf "*** begin: hexadecimal decoding process  ***\n"
	  end

	  tmp_sig = ""
	  tmp_sig_pos = 0
	  b = rsig.index('|',0)
	  e = rsig.index('|',b+1)

	  if b != 0
	    rsig[0..b-1].each_byte{|byte|
	      tmp_sig += " "
	      tmp_sig[tmp_sig_pos] = byte
	      tmp_sig_pos += 1
	    }
	  end
	  if $DEBUG
	    printf "*** after first parse ***\n"
	    printf "begin pos = (%d,%d)\n", b, e
	    printf "while_rsig= %s\n", rsig
	    printf "tmp_sig   = %s\n", tmp_sig

	    printf "rsig[b+1..e-1]\n"
	    printf "          = %s\n", rsig[b+1..e-1]
	  end
	  hex_sig = rsig[b+1..e-1].gsub(" ","")
	  i=0
	  while hex_sig != nil and i < hex_sig.size
	    if hex_sig[i] and hex_sig[i+1]
	      s = "  "
	      s[0] = hex_sig[i]
	      s[1] = hex_sig[i+1]
	      
	      tmp_sig += " "
	      tmp_sig[tmp_sig_pos] = s.hex
	      tmp_sig_pos += 1
	    end
	    i+=2
	  end

	  # next process
	  while e != rsig.size
	    b = rsig.index('|', e+1) # search next '|'
	    if $DEBUG
	      printf "*** next processing ***\n"
	      printf "next pos  = (%d)\n", b if b != nil
	    end
	    if b != nil # in the case, letters 'b' to next '|' is clear text.
	      rsig[e+1..b-1].each_byte{|byte|
		tmp_sig += " "
		tmp_sig[tmp_sig_pos] = byte
		tmp_sig_pos += 1
	      }

	      e = rsig.index('|', b+1)
	      hex_sig = rsig[b+1..e-1].gsub(" ","")
	      i=0
	      while hex_sig != nil and i < hex_sig.size
		if hex_sig[i] and hex_sig[i+1]
		  s = "  "
		  s[0] = hex_sig[i]
		  s[1] = hex_sig[i+1]
		  
		  tmp_sig += " "
		  tmp_sig[tmp_sig_pos] = s.hex
		  tmp_sig_pos += 1
		end
		i+=2
	      end
	      if $DEBUG
		printf "*** Final ***\n"
		printf "(b,e) = (%d,%d)\n", b, e
	      end
	    else
	      # there is no more '|' within left letters
	      rsig[e+1..rsig.size-1].each_byte{|byte|
		tmp_sig += " "
		tmp_sig[tmp_sig_pos] = byte
		tmp_sig_pos += 1
	      }
	      break
	    end
	  end

	  if $DEBUG
            print "space only: " if rsig =~ " "
	    print "BEFORE " + rsig + "\n"
	    print "AFTER  " + tmp_sig.dump + "\n"
	    print "*** end: hexadecimal decoding process  ***\n"
	  end
	  raise "error: wrong signature format is detected" if tmp_sig == ""
	  rsig = tmp_sig if tmp_sig != nil 
	end # finish (when signature contains the hex format strings)

	# produce quoted signature strings
	rsig = Regexp::new(Regexp::escape(rsig))
	# produce 'ruby-sig' form
	if is_renge
	  if @renge_sigs[dp] == nil
	    @renge_sigs[dp] = {sp=>[rsig]}
	  else
	    if @renge_sigs[dp][sp] == nil
	      @renge_sigs[dp][sp] = [rsig]
	    else
	      @renge_sigs[dp][sp] << rsig
	    end
	  end
	else
	  if @tcp_sigs[dp] == nil
	    @tcp_sigs[dp] = {sp=>[rsig]}
	  else
	    if @tcp_sigs[dp][sp] == nil
	      @tcp_sigs[dp][sp] = [rsig]
	    else
	      @tcp_sigs[dp][sp] << rsig
	    end
	  end
	end
	
	@tcp_sigs_maxlen = @tcp_sigs_maxlen < rsig.source.size ? rsig.source.size : @tcp_sigs_maxlen
	if $DEBUG
	  print rsig.source.dump + "\n"
	end
      }
    else
      raise "cannot read the file #{file}"
    end
    if $DEBUG
      printf "begin: signature analysis\n"
      p @tcp_sigs.each{|key,value|
	print key
	print " => "
	p value
      }
      
      @tcp_sigs.each_pair {|s,val|
	cnt = 0
	val.each_value{|v|
	  v.each{|k|
	    cnt+=1
	    printf "sig is %s\n", k.source.dump
	  }
	}
	printf "%d\n", cnt
      }

      # to produce lovely graph
      printf "produceing data for lovely graph\n"
      @tcp_sigs.each_pair {|s,val|
	printf "%s", s
	val.each_pair{|t,v|
	  printf "\t:%s", t
	  flag = 1
	  v.each{|k|
	    if flag == 1
	      printf "\t:%s\n", k.source.dump
	      flag = 0 
	    else
	      printf "\t:\t:%s\n", k.source.dump
	    end
	  }
	}
      }
      printf "end: signature analysis\n"

      printf "produceing data for lovely graph(2)\n"
      i = 0
      @tcp_sigs.each_pair {|s,val|
	i+= 10
	j = 0
	val.each_pair{ |t,v|
	  printf "%d ", i+j
	  j += 1
	  num = 0
	  v.each{ |k|
	    num += 1
	  }
	  printf "%d\n", num
	}
      }
      printf "end: signature analysis(2)\n"
    end
  end
end

#
# MAIN proc
#

def usage()
  printf " welcome patemon world\n"
  printf " usage: #{$0} <rule file>\n"
  printf " <rule file>: pakemon signature file compatible\n"
  printf " note: you need root privilege\n\n"
  exit
end

if ARGV[0] == nil 
  print "no input file\n\n"
  usage
end

if ! FileTest.exist?(ARGV[0])
  ptint "wrong input file\n\n"
end

trap("SIGHUP", "EXIT")
trap("SIGINT", "EXIT")

patimon = PATIMON.new.setup
patimon.read_sigs(ARGV[0])
patimon.use_kill false
patimon.run # for lan!

# end

