/**
 *
 * nids.c: there are rnids main proce
 * Copyright (c) 2000,2001 Yasuhiro ABE <yasu@dengaku.org>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **/

#include"rnids.h"

VALUE cNIDS; /* define class */
VALUE cuNIDS; /* current reference to object */
VALUE eNidsError;

struct stream_object {
  int is_kill; /*boolean*/
  int is_ip; /*boolean*/
  int is_udp; /*boolean*/
  int is_tcp; /*boolean*/
};

#define GetStream(obj, stream) \
    Data_Get_Struct(obj, struct stream_object, stream)

static void
free_stream(stream)
     struct stream_object *stream;
{
  free(stream);
}

void
ip_callback(iph)
     struct ip *iph;
{
  VALUE ret;
  ret = rb_funcall(cuNIDS, rb_intern("callback"), 1,
		   rnids_ippkt_new(iph));
}

void
udp_callback(addr, buf, len, iph)
     struct tuple4 *addr;
     char *buf;
     int len;
     struct ip *iph;
{
  VALUE ret;
  ret = rb_funcall(cuNIDS, rb_intern("callback"), 1,
		   rnids_udppkt_new(addr, buf, len, iph));
}

void
tcp_callback (a_tcp, conn_opt) /* aka. new_tcp_callback() */
     struct tcp_stream *a_tcp;
     void ** conn_opt;
{
  VALUE ret;
  int op_side = 0;
  struct stream_object *stream;

  GetStream(cuNIDS, stream);

  if (a_tcp->nids_state == NIDS_JUST_EST) { /* NIDS_JUST_EST */
    a_tcp->client.collect++;
    a_tcp->server.collect++;
    ret = rb_funcall(cuNIDS, rb_intern("callback"), 1,
		     rnids_tcppkt_new(a_tcp, NULL));
    return;
  }
  if (a_tcp->nids_state == NIDS_DATA) { /* NIDS_DATA */
    VALUE tcppkt;
    struct half_stream *hlf;
    if (a_tcp->client.count_new) {
      hlf = &a_tcp->client;
      op_side = 0;
    } else {
      hlf = &a_tcp->server;
      op_side = 1;
    }
    /**
     * disconnect this session
     **/
    tcppkt = rnids_tcppkt_new2(a_tcp, hlf, op_side);
    if(stream != NULL && stream->is_kill) {
      ret = rb_funcall(cuNIDS, rb_intern("kill"), 1, tcppkt);
      if(ret == Qtrue) {
	nids_killtcp(a_tcp);
      }
    }
    /**
     * callback function call
     **/
    ret = rb_funcall(cuNIDS, rb_intern("callback"), 1,tcppkt);
    nids_discard(a_tcp, hlf->count - hlf->offset);
  } else { /** NIDS_CLOSE, NIDS_RESET, NIDS_TIMED_OUT **/
    ret = rb_funcall(cuNIDS, rb_intern("callback"), 1,
		     rnids_tcppkt_new(a_tcp, NULL));
  }
}

static VALUE
rnids_s_new(argc, argv, klass)
     int argc;
     VALUE argv[];
     VALUE klass;
{
  struct stream_object *stream;
  VALUE obj;
  
  obj = Data_Make_Struct(klass, struct stream_object, 0, free_stream, stream);
  stream->is_tcp = 1;  /* == true */
  stream->is_udp = 0;  /* == false */
  stream->is_ip = 0;   /* == false */
  stream->is_kill = 1; /* == true */

  return obj;
}

/**
static VALUE
rnids_run2(self)
     VALUE self;
{
  struct stream_object *stream;
  int fd;
  int time = 0;
  fd_set rset;
  struct timeval tv;

  GetStream(self, stream);

  if (!nids_init ())
    {
      fprintf(stderr,"%s\n",nids_errbuf);
      exit(1);
    }
  //nids_register_tcp (tcp_callback);
    cuNIDS = self; // tell current object as global variable 
  if(stream->is_tcp) {
    nids_register_tcp(tcp_callback);
  }
  if(stream->is_udp) {
    nids_register_udp(udp_callback);
  }
  if(stream->is_ip) {
    nids_register_ip(ip_callback);
  }
  fd = nids_getfd();
  for (;;)
    {
      tv.tv_sec = 1;
      tv.tv_usec = 0;
      FD_ZERO (&rset);
      FD_SET (fd, &rset);
      // add any other fd we need to take care of
      if (select (fd + 1, &rset, 0, 0, &tv)) {
	if (FD_ISSET(fd,&rset)) {  // need to test it if there are other
	   cuNIDS = self;
	   TRAP_BEG;
	   if(!nids_next()){break;}
	   TRAP_END;
	}
      }	else {
	rb_thread_wait_fd(fd);
      }
    }
  return self;
}
**/

static VALUE
rnids_run(self)
     VALUE self;
{
  struct stream_object *stream;
  int ret;
  int fd;
  fd_set rset;
  struct timeval tv;

  GetStream(self, stream);

  if (!nids_init()) {
    fprintf(stderr,"%s\n", nids_errbuf);
    exit(1);
  }
  cuNIDS = self; /* tell current object as global variable */
  if(stream->is_tcp) {
    nids_register_tcp(tcp_callback);
  }
  if(stream->is_udp) {
    nids_register_udp(udp_callback);
  }
  if(stream->is_ip) {
    nids_register_ip(ip_callback);
  }
  fd = nids_getfd();

  FD_ZERO(&rset);
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  for(;;)
    {
      do {
	FD_SET(fd, &rset);
	if (select(fd+1, &rset, NULL, NULL, &tv) == 0) {
	  rb_thread_wait_fd(fd);
	}
	/* need to test it if there are other */
	if (FD_ISSET(fd,&rset)) {
	  cuNIDS = self;
	  TRAP_BEG;
	  ret = nids_next();
	  TRAP_END;
	}
      } while(ret); /* 'ret == 1' means success  */
      if (ret == 0) { /* when error */
	break;
      }
    }
  return self;
}


static VALUE
rnids_kill(self, state)
     VALUE self, state;
{
  return Qfalse;
}


static VALUE
rnids_tcp_stream(argc, argv, obj)
     int argc;
     VALUE argv[];
     VALUE obj;
{
  struct stream_object *stream;
  VALUE val;

  GetStream(obj, stream);
  
  if(argc == 0)
    return stream->is_tcp ? Qtrue : Qfalse;
  else
    val = argv[0];

  switch(TYPE(val))
    {
    case T_TRUE:
      stream->is_tcp = 1;
      break;
    case T_FALSE:
      stream->is_tcp = 0;
      break;
    }
  return obj;
}

static VALUE
rnids_udp_stream(argc, argv, obj)
     int argc;
     VALUE argv[];
     VALUE obj;
{
  struct stream_object *stream;
  VALUE val;

  GetStream(obj, stream);
  
  if(argc == 0)
    return stream->is_udp ? Qtrue : Qfalse;

  val = argv[0];

  switch(TYPE(val))
    {
    case T_TRUE:
      stream->is_udp = 1;
      return obj;
      break;
    case T_FALSE:
      stream->is_udp = 0;
      return obj;
      break;
    }
  return stream->is_udp ? Qtrue : Qfalse;
}

static VALUE
rnids_ip_stream(argc, argv, obj)
     int argc;
     VALUE argv[];
     VALUE obj;
{
  struct stream_object *stream;
  VALUE val;

  GetStream(obj, stream);
  
  if(argc == 0)
    return stream->is_ip ? Qtrue : Qfalse;

  val = argv[0];

  switch(TYPE(val))
    {
    case T_TRUE:
      stream->is_ip = 1;
      return obj;
    case T_FALSE:
      stream->is_ip = 0;
      return obj;
    }
  return stream->is_ip ? Qtrue : Qfalse;
}

/*kill*/
static VALUE
rnids_use_kill(argc, argv, obj)
     int argc;
     VALUE argv[];
     VALUE obj;
{
  struct stream_object *stream;
  VALUE val;

  GetStream(obj, stream);
  
  if(argc == 0)
    return stream->is_kill ? Qtrue : Qfalse;

  val = argv[0];

  switch(TYPE(val))
    {
    case T_TRUE:
      stream->is_kill = 1;
      return obj;
    case T_FALSE:
      stream->is_kill = 0;
      return obj;
    }
  return stream->is_kill ? Qtrue : Qfalse;
}

static VALUE
rnids_callback(self,val)
     VALUE self,val;
{
  return Qtrue;
}

/**
 * Ruby Initialize Function
 */
void
Init_rnids()
{
  cNIDS = rb_define_class("NIDS", rb_cObject);

  rb_define_const(cNIDS, "NIDS_JUST_EST", INT2NUM(NIDS_JUST_EST));
  rb_define_const(cNIDS, "NIDS_DATA", INT2NUM(NIDS_DATA));
  rb_define_const(cNIDS, "NIDS_CLOSE", INT2NUM(NIDS_CLOSE));
  rb_define_const(cNIDS, "NIDS_RESET", INT2NUM(NIDS_RESET));
  rb_define_const(cNIDS, "NIDS_TIMED_OUT", INT2NUM(NIDS_TIMED_OUT));

  rb_define_method(cNIDS, "run", rnids_run, 0);
  rb_define_method(cNIDS, "callback", rnids_callback, 1);

  rb_define_singleton_method(cNIDS, "new", rnids_s_new, -1);
  rb_define_method(cNIDS, "kill", rnids_kill, 1);
  rb_define_method(cNIDS, "use_tcp", rnids_tcp_stream, -1);
  rb_define_method(cNIDS, "use_udp", rnids_udp_stream, -1);
  rb_define_method(cNIDS, "use_ip", rnids_ip_stream, -1);
  rb_define_method(cNIDS, "use_kill", rnids_use_kill, -1);

  /*rb_define_variable("$CURRENT", &cuNIDS);*/
  rb_global_variable(&cuNIDS);

  eNidsError = rb_define_class_under(cNIDS, "NidsError", rb_eStandardError);

  /*Init_packet();*/
  Init_ippkt();
  Init_tcppkt();
  Init_udppkt();

  Init_support();
}
