/**
 *
 * support.c: NIDSupport definition of rnids support class 
 * Copyright (c) 2000,2001 Yasuhiro ABE <yasu@dengaku.org>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **/

#include"rnids.h"

static VALUE cNIDSupport;
ID id_ipfil;

/**
 * val: int promisc 
 * desc: non-zero, promics mode
 * default: 1
 **/
static VALUE
sp_promisc(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return nids_params.promisc ? Qtrue : Qfalse;
  }

  switch(argv[0])
    {
    case Qtrue:
      nids_params.promisc = 1;
      break;
    case Qfalse:
      nids_params.promisc = 0;
      break;
    default:
      nids_params.promisc = 0;
      break;
    }
  return self;
}
/**
 * val: int scan_delay
 * desc: set the scan delay time [ms]
 * default: 3000
 **/
static VALUE
sp_scan_delay(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.scan_delay);
  }

  nids_params.scan_delay = NUM2INT(argv[0]);
  return self;
}

/**
 * val: char *device
 * desc: set the listened device name
 * default: NULL
 **/
static VALUE
sp_device(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    if(nids_params.device==NULL) {
      return rb_str_new2("");
    } else {
      return rb_str_new2(nids_params.device);
    }
  }

  if(TYPE(argv[0]) == T_STRING) {
    if(RSTRING(argv[0])->len == 0) {
      nids_params.device = NULL;
    } else {
      nids_params.device = STR2CSTR(argv[0]);
    } 
  } else {
    rb_raise(rb_eTypeError, "String is required");
  }
  return self;
}

/**
 * val: char *pacp_filter
 * desc: set the link layer directive
 * default: NULL
 **/
static VALUE
sp_pcap_filter(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    if(nids_params.pcap_filter == NULL) {
      return rb_str_new2("");
    } else {
      return rb_str_new2(nids_params.pcap_filter);
    }
  }

  if(TYPE(argv[0]) == T_STRING) {
    if(RSTRING(argv[0])->len == 0) {
      nids_params.pcap_filter = NULL;
    } else {
      nids_params.pcap_filter = STR2CSTR(argv[0]);
    }
  } else {
    rb_raise(rb_eTypeError, "String is required");
  }
  return self;
}
/**
 * val: int sk_buff_size
 * desc: set the queuing packet buffer size
 * default: 168
 **/
static VALUE
sp_sk_buff_size(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.sk_buff_size);
  }

  nids_params.sk_buff_size = NUM2INT(argv[0]);
  return self;
}

/**
 * val: int scan_num_ports
 * desc: scanning ports number from the same source address
 * default: 10
 **/
static VALUE
sp_scan_num_ports(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.scan_num_ports);
  }

  nids_params.scan_num_ports = NUM2INT(argv[0]);
  return self;
}

/**
 * val: int scan_num_hosts
 * desc: hosts hash size for detecting the port-scan
 * default: 256
 **/
static VALUE
sp_scan_num_hosts(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.scan_num_hosts);
  }

  nids_params.scan_num_hosts = NUM2INT(argv[0]);
  return self;
}

/**
 * val: int dev_addon
 * desc: size in structure sk_buff is reserved 
 * default: -1
 **/
static VALUE
sp_dev_addon(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.dev_addon);
  }

  nids_params.dev_addon = NUM2INT(argv[0]);
  return self;
}

/**
 * val: int n_hosts
 * desc: hosts hash size for IP defragmentation
 * default: 256
 **/
static VALUE
sp_n_hosts(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.n_hosts);
  }

  nids_params.n_hosts = NUM2INT(argv[0]);
  return self;
}

/**
 * val: int n_tcp_streams
 * desc: hash size for tcp stream assembles
 * default: 1040 (if 0, not assemble tcp streams)
 **/
static VALUE
sp_n_tcp_streams(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.n_tcp_streams);
  }

  nids_params.n_tcp_streams = NUM2INT(argv[0]);
  return self;
}
/**
 * val: int (*ip_filter)(struct ip*)
 * desc: ip filtering function, retrun the non-zero value, 
 *       then the packet will be processed.
 * default: (always) return 1
 **/
static VALUE
sp_external_ip_filter(self, args)
     VALUE self, args;
{
  /* ruby function */
  return Qtrue;
}
static int 
sp_default_ip_filter(iph)
     struct ip *iph;
{
  /* called by libnids */
  return 1;
}
static int
sp_internal_ip_filter(iph)
     struct ip *iph;
{
  /* called by libnids  */
  VALUE ret;
  ret = rb_funcall(cNIDSupport, id_ipfil, 1, 
		   rnids_ippkt_new(iph));

  if(ret == Qtrue) {
    return 1;
  }
  return 0;
}
static VALUE
sp_ip_filter(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  static int ipfilag = Qfalse;
  if(argc == 0) {
    return ipfilag;
  }
  switch(argv[0])
    {
    case Qtrue:
      ipfilag = Qtrue;
      nids_params.ip_filter = sp_internal_ip_filter;
      break;
    case Qfalse:
      ipfilag = Qfalse;
      nids_params.ip_filter = sp_default_ip_filter;
      break;
    default:
      ipfilag = Qfalse;
      break;
    }
  return self;
}

/**
 * val: int syslog_level
 * desc: default syslog function is used
 * default: LOG_ALERT
 **/
static VALUE
sp_syslog_level(argc, argv, self)
     int argc;
     VALUE argv[];
     VALUE self;
{
  if(argc == 0) {
    return INT2NUM(nids_params.syslog_level);
  }

  nids_params.syslog_level = NUM2INT(argv[0]);
  return self;
}

void
Init_support()
{
  cNIDSupport = rb_define_module("NIDSupport");
  rb_define_module_function(cNIDSupport, "promisc", sp_promisc, -1);
  rb_define_module_function(cNIDSupport, "scan_delay", sp_scan_delay, -1);
  rb_define_module_function(cNIDSupport, "device", sp_device, -1);
  rb_define_module_function(cNIDSupport, "pcap_filter", sp_pcap_filter, -1);
  rb_define_module_function(cNIDSupport, "sk_buff_size", sp_sk_buff_size, -1);
  rb_define_module_function(cNIDSupport, "scan_num_ports", sp_scan_num_ports, -1);
  rb_define_module_function(cNIDSupport, "scan_num_hosts", sp_scan_num_hosts, -1);
  rb_define_module_function(cNIDSupport, "dev_addon", sp_dev_addon, -1);
  rb_define_module_function(cNIDSupport, "n_hosts", sp_n_hosts, -1);
  rb_define_module_function(cNIDSupport, "n_tcp_streams", sp_n_tcp_streams, -1);
  rb_define_module_function(cNIDSupport, "ip_filter", sp_ip_filter, -1);
  rb_define_module_function(cNIDSupport, "ipfilter", sp_external_ip_filter, 1);
  id_ipfil = rb_intern("ipfilter");

  rb_define_module_function(cNIDSupport, "syslog_level", sp_syslog_level, -1);

  rb_define_alias(cNIDSupport, "hash_hosts_size", "n_hosts");
  rb_define_alias(cNIDSupport, "hash_streams_size", "n_tcp_streams");
  rb_define_alias(cNIDSupport, "promiscuous", "promisc");
  rb_define_alias(cNIDSupport, "syslog_priority", "syslog_level");
}
