/**
 *
 * tcppkt.c: NIDS_TCP class definition 
 * Copyright (c) 2000,2001 Yasuhiro ABE <yasu@dengaku.org>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **/

#include"rnids.h"

VALUE cTCPPacket;

struct tcp_object {
  struct tcp_stream *a_tcp;
  struct half_stream *hlf;
  int op_side;
};

#define GetTCP(obj, tcph) \
    Data_Get_Struct(obj, struct tcp_object, tcph)

VALUE
rnids_tcppkt_new(a_tcp, hlf)  
     struct tcp_stream *a_tcp;
     struct half_stream *hlf;
{
  VALUE object;
  struct tcp_object *tcph;

  object = Data_Make_Struct(cTCPPacket, struct tcp_object, 0, free, (void *)tcph);
  tcph->a_tcp = a_tcp;
  tcph->hlf = hlf;
  return object;
}

VALUE
rnids_tcppkt_new2(a_tcp, hlf, op_side)
     struct tcp_stream *a_tcp;
     struct half_stream *hlf;
     int op_side;
{
  VALUE object;
  struct tcp_object *tcph;

  object = rnids_tcppkt_new(a_tcp, hlf);

  GetTCP(object, tcph);
  tcph->op_side = op_side;

  return object;
}

static VALUE
tcppkt_s_new(self,val)
     VALUE self, val;
{
  VALUE object;
  struct tcp_object *tcph;

  switch(TYPE(val)) {
  case T_DATA:
    GetTCP(val, tcph);
    break;
  default:
    rb_raise(rb_eTypeError, "TCP object is required");
  }

  object = Data_Wrap_Struct(cTCPPacket, 0, 0, (void *)tcph);
  return object;
}

static VALUE
tcppkt_to_s(self)
     VALUE self;
{
  struct tcp_object *tcph;
  
  GetTCP(self,tcph);
  
  if(tcph->op_side) {
    return rb_ary_new3(3,rb_str_new2(int_ntoa(tcph->a_tcp->addr.saddr)),
		       rb_str_new2(" -> "),
		       rb_str_new2(int_ntoa(tcph->a_tcp->addr.daddr)));
  } else {
    return rb_ary_new3(3,rb_str_new2(int_ntoa(tcph->a_tcp->addr.daddr)),
		       rb_str_new2(" -> "),
		       rb_str_new2(int_ntoa(tcph->a_tcp->addr.saddr)));
  }
}

static VALUE
tcppkt_opside(self)
     VALUE self;
{
  struct tcp_object *tcph;
  GetTCP(self, tcph);

  if(tcph->op_side > 0) {
    return Qtrue;
  } else {
    return Qfalse;
  }
}

static VALUE
tcppkt_killme(self)
     VALUE self;
{
  struct tcp_object *tcph;
  GetTCP(self, tcph);

  if(tcph->a_tcp != NULL) {
    nids_killtcp(tcph->a_tcp);
  }
  return self;
}

#define TCPP_METHOD(func, hlf_val, val) \
static VALUE\
(func)(self)\
     VALUE self;\
{\
    struct tcp_object *tcph;\
\
    GetTCP(self, tcph);\
    if(tcph->hlf) \
      return (hlf_val); \
    return (val);\
}

TCPP_METHOD(tcppkt_tcp_ack, INT2FIX(tcph->hlf->acked), INT2FIX(0))
TCPP_METHOD(tcppkt_tcp_seq, INT2FIX(tcph->hlf->seq), INT2FIX(0))
TCPP_METHOD(tcppkt_tcp_sport, INT2FIX(tcph->a_tcp->addr.source), INT2FIX(tcph->a_tcp->addr.source))
TCPP_METHOD(tcppkt_tcp_dport, INT2FIX(tcph->a_tcp->addr.dest), INT2FIX(tcph->a_tcp->addr.dest))
TCPP_METHOD(tcppkt_tcp_win, INT2FIX(tcph->hlf->window), INT2FIX(0))
TCPP_METHOD(tcppkt_tcp_saddr, rb_str_new2(int_ntoa(tcph->a_tcp->addr.saddr)), rb_str_new2(int_ntoa(tcph->a_tcp->addr.saddr)))
TCPP_METHOD(tcppkt_tcp_daddr, rb_str_new2(int_ntoa(tcph->a_tcp->addr.daddr)), rb_str_new2(int_ntoa(tcph->a_tcp->addr.daddr)))
TCPP_METHOD(tcppkt_tcp_state, INT2FIX(tcph->a_tcp->nids_state), INT2FIX(tcph->a_tcp->nids_state))
TCPP_METHOD(tcppkt_tcp_data, rb_tainted_str_new(tcph->hlf->data, tcph->hlf->count_new), rb_str_new2(""))
TCPP_METHOD(tcppkt_tcp_data_len, INT2FIX(tcph->hlf->count_new), INT2FIX(0))

void 
Init_tcppkt() {
  cTCPPacket = rb_define_class("NIDS_TCP", rb_cObject);
  /*rb_define_singleton_method(cTCPPacket, "new", tcppkt_s_new, 1);*/
  rb_define_method(cTCPPacket, "to_s", tcppkt_to_s, 0);

  rb_define_method(cTCPPacket, "tcp_data", tcppkt_tcp_data, 0);
  rb_define_method(cTCPPacket, "tcp_data_len", tcppkt_tcp_data_len, 0);
  rb_define_method(cTCPPacket, "tcp_dport", tcppkt_tcp_dport, 0);
  rb_define_method(cTCPPacket, "tcp_sport", tcppkt_tcp_sport, 0);
  rb_define_method(cTCPPacket, "tcp_ack", tcppkt_tcp_ack, 0);
  rb_define_method(cTCPPacket, "tcp_seq", tcppkt_tcp_seq, 0);
  rb_define_method(cTCPPacket, "tcp_win", tcppkt_tcp_win, 0);
  rb_define_method(cTCPPacket, "tcp_daddr", tcppkt_tcp_daddr, 0);
  rb_define_method(cTCPPacket, "tcp_saddr", tcppkt_tcp_saddr, 0);
  rb_define_method(cTCPPacket, "tcp_state", tcppkt_tcp_state, 0);
  
  rb_define_method(cTCPPacket, "tcp_direction?", tcppkt_opside, 0);
  rb_define_alias(cTCPPacket, "opposite_direction?", "tcp_direction?");
  rb_define_method(cTCPPacket, "tcp_kill", tcppkt_killme, 0);
  rb_define_alias(cTCPPacket, "kill", "tcp_kill");

  rb_define_alias(cTCPPacket, "state", "tcp_state");
  rb_define_alias(cTCPPacket, "daddr", "tcp_daddr");
  rb_define_alias(cTCPPacket, "saddr", "tcp_saddr");
  rb_define_alias(cTCPPacket, "dport", "tcp_dport");
  rb_define_alias(cTCPPacket, "sport", "tcp_sport");
  rb_define_alias(cTCPPacket, "data", "tcp_data");
  rb_define_alias(cTCPPacket, "data_len", "tcp_data_len");
}




