/**
 *
 * udppkt.c: NIDS_UDP class definition 
 * Copyright (c) 2000,2001 Yasuhiro ABE <yasu@dengaku.org>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 **/

#include"rnids.h"

VALUE cUDPPacket;

struct udp_object {
  struct tuple4 *addr;
  char *buf;
  int len;
  struct ip *iph;
};

#define GetUDP(obj, udph) \
    Data_Get_Struct(obj, struct udp_object, udph)

VALUE
rnids_udppkt_new(addr, buf, len, iph)
     struct tuple4 *addr;
     char *buf;
     int len;
     struct ip *iph;
{
  VALUE object;
  struct udp_object *udph;
  
  object = Data_Make_Struct(cUDPPacket, struct udp_object, 0, free, (void *)udph);
  udph->addr = addr;
  udph->buf = buf;
  udph->len = len;
  udph->iph = iph;

  return object;
}

static VALUE
udppkt_s_new(self,val)
     VALUE self, val;
{
  VALUE object;
  struct udp_object* udph;

  switch(TYPE(val)) {
  case T_DATA:
    GetUDP(val, udph);
    break;
  default:
    rb_raise(rb_eTypeError, "UDP object is required");
  }

  object = Data_Wrap_Struct(cUDPPacket, 0, free, (void *)udph);
  return object;
}

static VALUE
udppkt_to_s(self)
     VALUE self;
{
  struct udp_object *udph;
  
  GetUDP(self,udph);
  
  return rb_str_new2(int_ntoa(udph->iph->ip_src.s_addr));
}

#define UDP_METHOD(func, val) \
static VALUE\
(func)(self)\
     VALUE self;\
{\
    struct udp_object *udph;\
\
    GetUDP(self, udph);\
    return (val);\
}

UDP_METHOD(udppkt_data, rb_tainted_str_new(udph->buf, udph->len))
UDP_METHOD(udppkt_data_len, INT2FIX(udph->len))
UDP_METHOD(udppkt_get_ippkt, rnids_ippkt_new(udph->iph))
UDP_METHOD(udppkt_dport, INT2FIX(udph->addr->dest))
UDP_METHOD(udppkt_sport, INT2FIX(udph->addr->source))
UDP_METHOD(udppkt_daddr, rb_str_new2(int_ntoa(udph->addr->daddr)))
UDP_METHOD(udppkt_saddr, rb_str_new2(int_ntoa(udph->addr->saddr)))

void
Init_udppkt() {
  cUDPPacket = rb_define_class("NIDS_UDP", rb_cObject);
  /*rb_define_singleton_method(cUDPPacket, "new", udppkt_s_new, 1);*/

  rb_define_method(cUDPPacket, "to_s", udppkt_to_s, 0);
  rb_define_method(cUDPPacket, "udp_data", udppkt_data, 0);
  rb_define_method(cUDPPacket, "udp_data_len", udppkt_data_len, 0);
  rb_define_method(cUDPPacket, "get_ippkt", udppkt_get_ippkt, 0);
  rb_define_method(cUDPPacket, "udp_dport", udppkt_dport, 0);
  rb_define_method(cUDPPacket, "udp_sport", udppkt_sport, 0);
  rb_define_method(cUDPPacket, "udp_daddr", udppkt_daddr, 0);
  rb_define_method(cUDPPacket, "udp_saddr", udppkt_saddr, 0);

  rb_define_alias(cUDPPacket, "saddr", "udp_saddr");
  rb_define_alias(cUDPPacket, "daddr", "udp_daddr");
  rb_define_alias(cUDPPacket, "sport", "udp_sport");
  rb_define_alias(cUDPPacket, "dport", "udp_dport");
  rb_define_alias(cUDPPacket, "data", "udp_data");
  rb_define_alias(cUDPPacket, "data_len", "udp_data_len");
}
